@@ -1,9 +1,10 @@
 #ifndef __CR_NS_H__
 #define __CR_NS_H__
 
-#include "compiler.h"
+#include "common/compiler.h"
 #include "files.h"
-#include "list.h"
+#include "common/list.h"
+#include "images/netdev.pb-c.h"
 
 #ifndef CLONE_NEWNS
 #define CLONE_NEWNS	0x00020000
@@ -36,7 +37,8 @@
 #define CLONE_ALLNS	(CLONE_NEWPID | CLONE_NEWNET | CLONE_NEWIPC | CLONE_NEWUTS | CLONE_NEWNS | CLONE_NEWUSER | CLONE_NEWCGROUP)
 
 /* Nested namespaces are supported only for these types */
-#define CLONE_SUBNS	(CLONE_NEWNS)
+#define CLONE_SUBNS	(CLONE_NEWNS | CLONE_NEWNET)
+
 #define EXTRA_SIZE	20
 
 struct ns_desc {
@@ -70,6 +72,18 @@ enum ns_type {
 	NS_OTHER,
 };
 
+struct netns_id {
+	unsigned		target_ns_id;
+	unsigned		netnsid_value;
+	struct list_head	node;
+};
+
+struct net_link {
+	NetDeviceEntry		*nde;
+	bool			created;
+	struct list_head	node;
+};
+
 struct ns_id {
 	unsigned int kid;
 	unsigned int id;
@@ -84,19 +98,37 @@ struct ns_id {
 	 * are mounted) and other tasks may do setns on it
 	 * and proceed.
 	 */
-	futex_t ns_populated;
+	bool ns_populated;
 
 	union {
 		struct {
 			struct mount_info *mntinfo_list;
 			struct mount_info *mntinfo_tree;
-			int ns_fd;
-			int root_fd;
+			int nsfd_id;
+			int root_fd_id;
 		} mnt;
 
 		struct {
+
+			/*
+			 * ns_fd is used when network namespaces are being
+			 * restored. On this stage we access these file
+			 * descriptors many times and it is more efficient to
+			 * have them opened rather than to get them from fdstore.
+			 *
+			 * nsfd_id is used to restore sockets. On this stage we
+			 * can't use random file descriptors to not conflict
+			 * with restored file descriptors.
+			 */
+			union {
+				int nsfd_id;	/* a namespace descriptor id in fdstore */
+				int ns_fd;	/* a namespace file descriptor */
+			};
 			int nlsk;	/* for sockets collection */
 			int seqsk;	/* to talk to parasite daemons */
+			struct list_head ids;
+			struct list_head links;
+			NetnsEntry *netns;
 		} net;
 	};
 };
@@ -127,6 +159,7 @@ extern int prepare_namespace_before_tasks(void);
 extern int prepare_namespace(struct pstree_item *item, unsigned long clone_flags);
 
 extern int switch_ns(int pid, struct ns_desc *nd, int *rst);
+extern int switch_ns_by_fd(int nsfd, struct ns_desc *nd, int *rst);
 extern int restore_ns(int rst, struct ns_desc *nd);
 
 extern int dump_task_ns_ids(struct pstree_item *);
@@ -171,7 +204,7 @@ typedef int (*uns_call_t)(void *arg, int fd, pid_t pid);
  * If the operation in question can be offloaded to another process,
  * this call allows to do that.
  *
- * In case we're not in userns, just call the callback immediatelly
+ * In case we're not in userns, just call the callback immediately
  * in the context of calling task.
  */
 extern int __userns_call(const char *func_name, uns_call_t call, int flags,
@@ -183,4 +216,7 @@ extern int __userns_call(const char *func_name, uns_call_t call, int flags,
 
 extern int add_ns_shared_cb(int (*actor)(void *data), void *data);
 
+extern struct ns_id *get_socket_ns(int lfd);
+extern struct ns_id *lookup_ns_by_kid(unsigned int kid, struct ns_desc *nd);
+
 #endif /* __CR_NS_H__ */
