@@ -3,7 +3,7 @@
 
 #include "pid.h"
 #include "common/list.h"
-#include "config.h"
+#include "common/config.h"
 #include "asm/parasite-syscall.h"
 
 struct parasite_dump_thread;
@@ -22,7 +22,7 @@ struct rt_sigframe;
 
 struct parasite_ctl;
 
-extern int parasite_dump_sigacts_seized(struct parasite_ctl *ctl, struct cr_imgset *cr_imgset);
+extern int parasite_dump_sigacts_seized(struct parasite_ctl *ctl, struct pstree_item *);
 extern int parasite_dump_itimers_seized(struct parasite_ctl *ctl, struct pstree_item *);
 
 struct proc_posix_timers_stat;
