@@ -3,8 +3,8 @@
 
 #include <compel/compel.h>
 
+#include "common/config.h"
 #include "common/compiler.h"
-#include "config.h"
 
 #define pie_size(__pie_name)	(round_up(sizeof(__pie_name##_blob) + \
 			__pie_name ## _nr_gotpcrel * sizeof(long), page_size()))
